{% extends "django_cradmin/viewhelpers/formview/standalone_form_view.django.html" %}
{% load i18n %}

{% block title %}
    {% trans "Sign in" %}
{% endblock title %}

{% block page-cover-title %}
    {% trans "Sign in" %}
{% endblock page-cover-title %}

{% block global-bodyclass %}
login
{% endblock global-bodyclass %}

{% block content %}
    {{ block.super }}

    {% if DJANGO_CRADMIN_FORGOTPASSWORD_URL %}
        <p id="cradmin_authenticate_login_forgotpasswordlink">
            <a href="{{ DJANGO_CRADMIN_FORGOTPASSWORD_URL }}">
                {% trans "Forgot your password?" %}
            </a>
        </p>
    {% endif %}
{% endblock content %}
