{% extends "cradmin_resetpassword/base.django.html" %}
{% load i18n %}
{% load cradmin_tags %}

{% block subtitle %}
    {% trans "Check your email" %}
{% endblock subtitle %}

{% block page-cover-title %}
    {% trans "Check your email" %}
{% endblock page-cover-title %}

{% block content %}
    <div class="adminui-page-section  adminui-page-section--tight{% cradmin_test_css_class 'page-content' %}">
        <p>
            {% trans "We have sent you an email. Click the link in the email to reset your password." %}
        </p>
        <p>
            {% blocktrans with email_subject=email_subject %}If you do not see the email, check your junk folder or try searching for "{{ email_subject }}".{% endblocktrans %}
        </p>
    </div>
{% endblock content %}
