{% extends "django_cradmin/crmenu/base.django.html" %}

{% block list-element-attributes %}
    cradmin_menutoggle_groupid="mainmenu"
    cradmin_menutoggle_activestyle="adminui-expandable-menu--expanded"
{% endblock list-element-attributes %}

{% block content %}
    <div class="adminui-expandable-menu__backdrop" cradmin_menutoggle_groupid="mainmenu" cradmin_menutoggle_clickable></div>
    <div class="adminui-expandable-menu__content">
        {{ block.super }}
    </div>
{% endblock content %}
