<table width="100%" border="0" cellspacing="0" cellpadding="0">
    <tr>
        <td>
            <table border="0" cellspacing="0" cellpadding="0" width="100%">
                <tr>
                    <td align="left" style="text-align: left;">
                        <a href="{{ url }}" target="_blank"
                           style="{{ linkstyle }}">{{ label }}</a>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>
