{% extends "django_cradmin/crheader/base.django.html" %}
{% load cradmin_tags %}

{% block outer-wrapper-element-attributes %}
    {{ block.super }}
    {% if me.include_menutoggle_javascript %}
        cradmin_menutoggle_groupid="mainmenu"
        cradmin_menutoggle_activestyle="{{ me.bem_block }}--showmenu"
    {% endif %}
{% endblock outer-wrapper-element-attributes %}

{% block content %}
    <div class="{{ me.bem_block }}__content">
        {% block content-inner %}
            {% block brand %}
                <a class="{{ me.bem_block }}__brand"
                        href="{% cradmin_instance_rolefrontpage_url %}">
                    {% block brand_content %}
                        <span class="{{ me.bem_block }}__brandname">
                            {% cradmin_titletext_for_role request.cradmin_role %}
                        </span>
                    {% endblock brand_content %}
                </a>
            {% endblock brand %}

            {% block main-menu %}
                {% if request.cradmin_instance.main_menu_renderable %}
                    {% cradmin_render_renderable request.cradmin_instance.main_menu_renderable %}
                {% endif %}
            {% endblock main-menu %}
        {% endblock content-inner %}
    </div>
{% endblock content %}
