# from __future__ import unicode_literals
#
# from django import forms
# from django import test
# from . import mixins
#
#
# class TestFieldWrapperRenderable(test.TestCase, mixins.SingleFormRenderableHelperMixin):
#     def test_stuff(self):
#         selector = self.single_field_formrenderable_htmls(field=forms.CharField())
#         selector.prettyprint()
