{% extends "django_cradmin/viewhelpers/objecttable/objecttable.django.html" %}
{% load cradmin_tags %}

{% block bodyclass %}
    {{ block.super }}
    django-cradmin-objecttable-filterlist-top-page
{% endblock %}


{% block tablewrapper %}
    {% cradmin_render_renderable filterlist %}
    {{ block.super }}
{% endblock tablewrapper %}
