{% extends "django_cradmin/layouts/standalone/focused.django.html" %}
{% load i18n %}

{% block page-cover-title %}
    {% trans "Permission denied" %}
{% endblock page-cover-title %}

{% block content %}
    <div class="adminui-page-section adminui-page-section--center-md">
        <div class="container container--tight">
            <p>
                {% blocktrans with what=role %}You do not have permission to administer "{{what}}".{% endblocktrans %}
            </p>
        </div>
    </div>
{% endblock content %}
