{% comment %}
    Base template for guide examples.
{% endcomment %}{% comment %}
    Base template for all django_cradmin templates.

    You should extend this directly for standalone pages like
    login and error pages. For pages that require a role,
    you should extend ``base.django.html`` instead.
{% endcomment %}{% spaceless %}
    {% load cradmin_javascriptregistry_tags %}
{% endspaceless %}<!DOCTYPE html>
<html {% block html_element_attributes %}lang="en"{% endblock %}>
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1">

        <title>{{ section.reference }} - Example {{ exampleindex }}</title>

        {% block styles %}
            <link href='{{ styleguideconfig.get_cssurl_for_styleguide }}' rel="stylesheet">
        {% endblock styles %}

        {% block javascript %}
            {# Override this to add javascript #}
        {% endblock javascript %}

        {% block javascriptregistry-head %}
            {% cradmin_javascriptregistry_head %}
        {% endblock javascriptregistry-head %}

    </head>
    <body>
        {{ example.text|safe }}
        {% block end-of-body %}{% endblock end-of-body %}
        {% block javascriptregistry-end-of-body %}
            {% cradmin_javascriptregistry_end_of_body %}
        {% endblock javascriptregistry-end-of-body %}
    </body>
</html>
