{% load cradmin_tags %}

{% block pre %}{% endblock pre %}
<{{ me.get_wrapper_htmltag }} {% if me.get_css_classes_string %}class="{{ me.get_css_classes_string }}"{% endif %} {% block list-element-attributes %}{% endblock %}>
    {% block content %}
        {% for renderable in me.iter_renderables %}
            {% block list-item %}
                {% cradmin_render_renderable renderable %}
            {% endblock list-item %}
        {% endfor %}
    {% endblock content %}
</{{ me.get_wrapper_htmltag }}>
{% block post %}{% endblock post %}
