/* This stylesheet generated by Transfonter (http://transfonter.org) on September 23, 2016 7:54 AM */

@font-face {
	font-family: 'Noe Display';
	src: url('NoeDisplay-Bold.eot');
	src: url('NoeDisplay-Bold.eot?#iefix') format('embedded-opentype'),
		url('NoeDisplay-Bold.woff2') format('woff2'),
		url('NoeDisplay-Bold.woff') format('woff'),
		url('NoeDisplay-Bold.ttf') format('truetype');
	font-weight: bold;
	font-style: normal;
}
