{% extends "django_cradmin/layouts/standalone/focused.django.html" %}
{% load i18n %}

{% block title %}
    {% trans "Signed out" %}
{% endblock title %}

{% block page-cover-title %}
    {% trans "Signed out" %}
{% endblock page-cover-title %}

{% block content %}
    <div class="adminui-page-section adminui-page-section--center">
        <a href="{{ LOGIN_URL }}" class="button">
            {% trans "Sign back in" %}
        </a>
    </div>

{% endblock content %}
