{% block outer_wrapper %}
    <{{ me.html_tag }}{% block outer_wrapper_attributes %}{{ me.html_element_attributes_string }}{% endblock %}>
        {% block content %}
            {% include "django_cradmin/uicontainer/includes/render_children.django.html" %}
        {% endblock content %}
    {% if me.html_tag_supports_children %}</{{ me.html_tag }}>{% endif %}
{% endblock outer_wrapper %}
