{% extends "django_cradmin/standalone-base-internal.django.html" %}
{% comment %}
    You can replace this template in your own project, and
    override stuff from standalone-base-internal.django.html.
    This means that you can put ``django_cradmin/standalone-base.django.html``
    in your own project, and override it without having to reimplement
    anything.

    .. note:: Make sure you put the Django app overriding this template
        after ``django_cradmin`` in ``INSTALLED_APPS``.

    .. note:: We will NEVER add anything to this template by default.
{% endcomment %}
