{% extends "django_cradmin/layouts/standalone/focused.django.html" %}
{% load cradmin_kss_styleguide_tags %}

{% block page-cover-title %}
    Style guides
{% endblock page-cover-title %}

{% block content %}
    <div class="adminui-page-section adminui-page-section--tight">
        <nav class="blocklist">
            {% for styleguideconfig in styleguideregistry.iterstyleguides %}
                <a class="blocklist__item blocklist__item--link"
                        href="{% url 'cradmin_kss_styleguide_guide' unique_id=styleguideconfig.unique_id %}">
                    {{ styleguideconfig.label }}
                </a>
            {% endfor %}
        </nav>
    </div>
{% endblock content %}
