{% extends "django_cradmin/layouts/standalone/focused.django.html" %}
{% load i18n %}
{% load static %}
{% load cradmin_tags %}

{% block title %}
    {{ pagetitle }}
{% endblock title %}

{% block page-cover-title %}
    {{ pagetitle }}
{% endblock page-cover-title %}

{% block page-cover-content %}
    {{ block.super }}
    <p>
        {% trans "Please select what you would like to edit" %}
    </p>
{% endblock page-cover-content %}

{% block content %}
    {% cradmin_render_renderable djangoapps_list %}
{% endblock content %}
