{% extends "django_cradmin/uicontainer/container.django.html" %}

{% block content %}
    {% block help_text %}
        {{ me.help_text }}
    {% endblock help_text %}
    {{ block.super }}
{% endblock content %}
