{% extends "django_cradmin/viewhelpers/listbuilder/base/list.django.html" %}

{% block list-element-attributes %}
    {% if me.get_wrapper_htmltag_id %}id="{{ me.get_wrapper_htmltag_id }}"{% endif %}
{% endblock list-element-attributes %}
