{% extends "cradmin_kss_styleguide/styleguideview/guide.django.html" %}
{% load static %}

{% block html_element_attributes %}
    {{ block.super }}
    ng-app="djangoCradminUi"
{% endblock html_element_attributes %}

{% block javascript %}
    <script src="{% static 'django_cradmin/dist/vendor/cradmin-vendorjs.js' %}" type="text/javascript"></script>
    <script src="{% static 'django_cradmin/dist/js/cradmin.min.js' %}" type="text/javascript"></script>
{% endblock javascript %}


{% block end-of-body %}
    <script>
        angular.module('djangoCradminUi', [
            'djangoCradmin'
        ]);
    </script>

{% endblock end-of-body %}
