{% extends "django_cradmin/base.django.html" %}
{% load i18n %}
{% load static %}
{% load cradmin_tags %}


{% block page-cover-title %}
    {{ djangoappconfig.get_menulabel }}
{% endblock page-cover-title %}

{% block content %}
    <div class="container">
        {% cradmin_render_renderable models_list %}
    </div>
{% endblock content %}
