{% load i18n %}
{% load cradmin_tags %}

{% if messages %}
    <div id="id_django_cradmin_messages" role="complementary" class="blocklist  blocklist--tight{% cradmin_test_css_class 'djangomessages' %}">
        {% for message in messages %}
            <p class="blocklist__item
                        {% if message.tags == "error" %}
                            blocklist__item--warning
                        {% elif message.tags == "warning" %}
                            blocklist__item--warning
                        {% elif message.tags == "info" %}
                            blocklist__item--info
                        {% elif message.tags == "success" %}
                            blocklist__item--success
                        {% endif %}
                        {% cradmin_test_css_class message.tags %}
                    "
                    role="alertdialog">
                <span class="screenreader-only">
                    {% if message.tags == "error" %}
                        {% trans "Error" %}
                    {% elif message.tags == "warning" %}
                        {% trans "Warning" %}
                    {% elif message.tags == "info" %}
                        {% trans "Info" %}
                    {% elif message.tags == "success" %}
                        {% trans "Success" %}
                    {% endif %}
                    :
                </span>
                {{ message }}
            </p>
        {% endfor %}
    </div>
{% endif %}
