{% extends "django_cradmin/layouts/standalone/focused.django.html" %}
{% load i18n %}

{% block page-cover-title %}
    {% trans "Invalid role" %}: {{ roleid }}
{% endblock page-cover-title %}

{% block content %}
    <p>{% trans "The requested role does not exist." %}</p>
{% endblock content %}
