{% extends "django_cradmin/base.django.html" %}
{% load i18n %}

{% block title %}
    {{ pagetitle }}
{% endblock title %}

{% block page-cover-title %}
    {{ pagetitle }}
{% endblock page-cover-title %}

{% block content %}
    <div class="adminui-page-section  adminui-page-section--center">
        {% block deletequestion %}
            <div id="id_deleteview_question">
                <p>{{ confirm_message }}</p>
            </div>
        {% endblock deletequestion %}
    </div>
    <div class="adminui-page-section  adminui-page-section--center">
        {% block delete-form %}
            <form action="{{ request.build_absolute_uri }}" method="post" class="django-cradmin-form">
                {% csrf_token %}
                <button type="submit" class="button  button--primary">{{ delete_button_label }}</button>
                <button type="button" class="button" onclick="history.back();return false;">{% trans "Cancel" %}</button>
            </form>
        {% endblock delete-form%}
    </div>
{% endblock content %}
